@@ -15,6 +15,8 @@
 #include "../smalloc.h"
 #include "../file.h"
 #include "../log.h"
+#include "../lib/hweight.h"
+#include "../lib/strcasestr.h"
 
 #include "windows/posix.h"
 
@@ -104,6 +106,7 @@ int fcntl(int fildes, int cmd, ...);
 int fdatasync(int fildes);
 int lstat(const char * path, struct stat * buf);
 uid_t geteuid(void);
+char* ctime_r(const time_t *t, char *buf);
 int nanosleep(const struct timespec *rqtp, struct timespec *rmtp);
 ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
 ssize_t pwrite(int fildes, const void *buf, size_t nbyte,
@@ -161,11 +164,6 @@ static inline unsigned long long os_phys_mem(void)
 	return (unsigned long long) pages * (unsigned long long) pagesize;
 }
 
-static inline void os_get_tmpdir(char *path, int len)
-{
-	GetTempPath(len, path);
-}
-
 static inline int gettid(void)
 {
 	return GetCurrentThreadId();
@@ -214,6 +212,16 @@ static inline void fio_cpu_set(os_cpu_mask_t *mask, int cpu)
 	*mask |= 1 << cpu;
 }
 
+static inline int fio_cpu_isset(os_cpu_mask_t *mask, int cpu)
+{
+	return (*mask & (1U << cpu));
+}
+
+static inline int fio_cpu_count(os_cpu_mask_t *mask)
+{
+	return hweight64(*mask);
+}
+
 static inline int fio_cpuset_init(os_cpu_mask_t *mask)
 {
 	*mask = 0;
